// HRPCFreqCal.c
// Mex function for hi-res performance counter frequency calibration
//
// Usage:
//
//      >> CalFreq = HRPCFreqCal(DT);
//      or
//      >> [CalFreq,NomFreq] = HRPCFreqCal(DT);
//
//      where
//      DT      calibration duration [s]
//      CalFreq Calibrated frequency [Hz]
//      NomFreq Nominal frequency [Hz] (just for comparison)
//
// V1.0 may/2007
// RP Marques
//
// Uses copyrighted material (see comments)

#include "mex.h"
#include "windows.h"

// TSC access function (see Copyright Note in the end of file)
__forceinline ULONGLONG gettsc(void) {
  ULONG hi,lo;
  LARGE_INTEGER r;
  __asm {
      rdtsc
      mov hi,edx
      mov lo,eax
  }
  r.HighPart = hi;
  r.LowPart = lo;
  return r.QuadPart;
}

// Main function
void HRPCFreqCal(double CalFreq[], double NomFreq[], double CalInterval[])
{
  LARGE_INTEGER Freq_int64;
  ULONGLONG basetsc,curtsc;  // in processor clocks
  FILETIME baseft,curft;     // in hecto-nanoseconds (100ns units, 10^-7 sec)
  ULONGLONG basest, curst;
  ULONGLONG deltsc, delst;
  
  // Nominal frequency
  if ( QueryPerformanceFrequency(&Freq_int64) == 0 ) {
      mexErrMsgTxt("Hi-res performance counter unavailable.");
  }
  NomFreq[0] = ( (double) Freq_int64.QuadPart ); // NomFreq in Hz
  
  // Start calibration
  
  /*******************************************/
  /* (see Copyright Note in the end of file) */
  // get base times 
  Sleep(5);  // to be sure we're on a tick boundary
  basetsc = gettsc();  // get this first because it changes very fast
  GetSystemTimeAsFileTime(&baseft);  // no rush; we have 1/64 sec to read this before it changes
  // get later times
  Sleep(( (DWORD) CalInterval[0] )*1e3);
  curtsc = gettsc();
  GetSystemTimeAsFileTime(&curft);
  // coerce filetimes
  basest = ((ULONGLONG) baseft.dwHighDateTime)<<32 | ((ULONGLONG) baseft.dwLowDateTime);
  curst = ((ULONGLONG) curft.dwHighDateTime)<<32 | ((ULONGLONG) curft.dwLowDateTime);

  // do calibration
  // NB: dwLowDateTime wraps every 429.5 seconds, so don't just use that!
  deltsc = curtsc - basetsc;
  delst = curst - basest;
  CalFreq[0] = ((double) deltsc) / ((( (double) delst)) / 1e7);  
  /********************************************/
}


// Gateway function
void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
                 const mxArray *prhs[])
{
  double *CalInterval, *CalFreq, *NomFreq;
  int mrows, ncols;
  
  /* Check for proper number of arguments. */
  if (nrhs != 1) {
    mexErrMsgTxt("One input required.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Too many output arguments");
  }
  
  /* The input must be a noncomplex scalar double.*/
  mrows = mxGetM(prhs[0]);
  ncols = mxGetN(prhs[0]);
  if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0]) ||
      !(mrows == 1 && ncols == 1)) {
    mexErrMsgTxt("Input must be a noncomplex scalar double.");
  }

  /* Create matrices for the return arguments. */
  plhs[0] = mxCreateDoubleMatrix(1,1, mxREAL);
  plhs[1] = mxCreateDoubleMatrix(1,1, mxREAL);
  
  /* Assign pointers to each input and output. */
  CalInterval = mxGetPr(prhs[0]);   // Calibration procedure duration
  CalFreq = mxGetPr(plhs[0]);       // Calibrated frequency
  NomFreq = mxGetPr(plhs[1]);       // Nominal frequency
  
  /* Call the HRPCFreqCal subroutine. */
  HRPCFreqCal(CalFreq,NomFreq,CalInterval);
}

// Notice on use of copyrighted material

// Copyright (c) 2003, Keith Wansbrough
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//     * Redistributions of source code must retain the above
//       copyright notice, this list of conditions and the following
//       disclaimer.
//     * Redistributions in binary form must reproduce the above
//       copyright notice, this list of conditions and the following
//       disclaimer in the documentation and/or other materials provided
//       with the distribution.
//     * Neither the name of the University of Cambridge nor the names of its
//       contributors may be used to endorse or promote products derived
//       from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
//

// (for information only: this is the BSD license as found on
//  http://www.opensource.org/licenses/bsd-license.php)
