/* =====================================================================*/
/*                                                                      */
/* file: IO_CRST.C                                                      */
/*
;-----------------------------------------------------------------------+
;  ANALOG DEVICES ADP-218X <===> Crystal Interconnects I/O Mapped       |
;-----------------------------------------------------------------------+
;-----------------------------------------------------------------------+
; CONTROLE DE VERSAO                                                    |
; Versao  Modificado por   		Descricao                               |
;   1.00  JAIME			   		Adaptacao da AC2122                     |
:   2.00  JAIME            		Endereco do Cristal A8/A7/A6/A0         |
:   2.01  JAIME            		Mensagem de troca do IP					|
;                          		IP4 e MAC6 default = 32                 |
;   2.02  RTT    09/apr/2006    Play Back                               |
;                                                                       |
;-----------------------------------------------------------------------+
; 1) Mensagens recebidas do Servidor (DSP)                              |
;    <header UDP><comando>                                              |
;    <comando> ::= comando a ser executado (1 byte):                    |
;       0x01: Comando Start                                             |
;             Memoriza endereo IP/Port do cliente e habilita envio de  |
;             pacotes de 1440 bytes a cada 5 ms aproximadamente para    |
;             o cliente. O bit 7 de flags  utilizado para indicar se a |
;             transmisso de pacotes de 1440 bytes est habilitada.     |
;                                                                       |
;       0x02: Comando Stop                                              |
;             Desabilita o envio de pacotes de 1440 bytes.              |
;                                                                       |
; 2) Formato do pacote de 1440 bytes enviados para o cliente.           |
;    <header UDP><nmero do pacote><dados>                              |
;    <nmero do pacote> ::= nmero do pacote (ulong - 4 bytes)          |
;    <dados> ::= vetor de 1440-4 bytes. O contedo do vetor  dado pela |
;                expresso:                                             |
;                dados[i] = (i+5) mod 256,   i = 0, 1, ...              |
;                                                                       |
;-----------------------------------------------------------------------+
*/


/*==============================================================*/
/*  Packet offset equates                                       */
/*  Packet header                                               */
/*==============================================================*/

#define pktLen     0x00
#define pktDest0   0x01
#define pktDest1   0x02
#define pktDest2   0x03
#define pktSrc0    0x04
#define pktSrc1    0x05
#define pktSrc2    0x06
#define pktType    0x07


/*--------------------------------------*/
/*  CPU CRYSTAL CLOCK                   */
/*                                      */

#define    ClkFreq  24000000    /* 24Mhz crystal=>DSP AT 48Mhz */

/*--------------------------------------*/
/*  IP                                  */
/*                                      */

#define    IP1_DEF	192         /* first octet of IP address   */
#define    IP2_DEF	168         /* second octet of IP address  */
#define    IP3_DEF	1           /* third octet of IP address   */
#define    IP4_DEF	32          /* fourth octet of IP address  */

/*--------------------------------------*/
/*  MAC                                 */
/*                                      */

/*--------------------------------------*/
/*  PORTs                               */
/*                                      */

#define    PortCmd 		11111   	/* Port UDP para recepcao de comando    */
#define    DSPX_Port    11115       /* Port UDP para acesso ao DSPX         */
#define    PlayBk_Port  11120       /* Port UDP para acesso ao DSPX         */

/*--------------------------------------*/
/*  ARP                                 */
/*                                      */

#define ar_hwtype   0x08        /* hardware type                            */
#define ar_prtype   0x09        /* protocol type                            */
#define ar_hwlen    0x0a        /* hardware address length                  */
                                /* and protocol address length              */
#define ar_op       0x0b        /* ARP operation (1=request, 2=reply)       */
#define ar_sha      0x0c        /* senders hardware address                 */
#define ar_spa      0x0f        /* senders IP address                       */
#define ar_tha      0x11        /* target hardware address                  */
#define ar_tpa      0x14        /* target IP address                        */

/*--------------------------------------*/
/* IP header                            */
/*                                      */

#define ip_verlen   0x08        /* IP version and header length(in longs)   */
						        /* and ip-tos IP type of service            */
#define ip_len      0x09        /* packet length (length-header_length)     */
#define ip_id       0x0a        /* datagram id                              */
#define ip_fragoff  0x0b        /* fragment offset                          */
#define ip_ttl      0x0c        /* time to live (in gateway hops)           */
						        /* and ip-proto protocol (ICMP=1, TCP=6, EGP=8, UDP=17)  */
#define ip_cksum    0x0d        /* header checksum                          */
#define ip_src      0x0e        /* IP address of source                     */
#define ip_dst      0x10        /* IP addess of destination                 */
#define ip_data     0x12

/*--------------------------------------*/
/*  IP value equates                    */
/*                                      */

#define IPT_ICMP    1           /* protocol type for ICMP packets  */
#define IPT_TCP     6           /* protocol type for TCP packets   */
#define IPT_EGP     8           /* protocol type for EGP packets   */
#define IPT_UDP     0x11        /* protocol type for UDP packets   */

/*--------------------------------------*/
/*  ICMP header                         */
/*                                      */

#define ic_type     ip_data     /* 0=reply, 8=request, others=who-cares */
								/* and ic_code                          */
#define ic_cksum    ic_type+1   /* checksum of header+data              */
#define ic_id       ic_cksum+1  /* message id                           */
#define ic_seq      ic_id+1     /* sequence number                      */
#define ic_data     ic_seq+1    /* optional data                        */

/*--------------------------------------*/
/*  UDP Header                          */
/*                                      */

#define u_src       ip_data     /* source udp port number      */
#define u_dst       u_src+1     /* destination UDP port number */
#define u_len       u_dst+1     /* length of UDP header+data   */
#define u_cksum     u_len+1     /* checksum  (see note)        */
#define u_data      u_cksum+1   /* start of data               */


/*--------------------------------------*/
/*  TFTP Header 	                    */
/*             		                    */

#define t_opcode	u_data	    /* TFTP opcode */
#define t_blk_nbr	t_opcode+1	/* TFTP block number */
#define t_err_nbr	t_opcode+1	/* TFTP error number */
#define t_filename	t_opcode+1	/* TFTP file name */
#define t_data	    t_blk_nbr+1	/* TFTP data */
#define t_err_msg	t_err_nbr+1	/* TFTP error message */


/*--------------------------------------*/
/*  AC2122 UDP message equates          */
/*                                      */

/*------------------------------------------*/
/*--- Codigos dos pacotes UDP do AC2120 --- */

/* 0x0100 a 0x01ff: reservados para o gerenciamento de condicionadores */

#define pcGetAI216x    0x0100       /* Comando de leitura de setup do AI2161/2164 */
#define pcGetAI216xR   0x0101       /* Resposta ao pcGetAI216x */
#define pcSetAI216x    0x0102       /* Comando de escrita de setup do AI2161/2164 */
#define pcSetAI216xR   0x0103       /* Resposta ao pcSetAI216x */

/* 0x0200 a 0x02ff: reservados para entradas analogicas */
/* 0x0300 a 0x03ff: reservados para entradas digitais   */

#define pcReadDI  	   0x0300 		/* Leitura de entrada digital */
#define pcReadDI_R 	   0x0301 		/* Resposta ao pcReadDI */

/* 0x0400 a 0x04ff: reservados para entradas de pulso */
/* 0x0500 a 0x05ff: reservados para saidas analogicas */
/* 0x0600 a 0x06ff: reservados para saidas digitais */

#define pcWriteDO  		0x0600 		/* Escrita em saida digital */
#define pcWriteDO_R		0x0601 		/* Resposta ao pcWriteDO */

/* 0x0700 a 0x07ff: reservados para o gerenciamento
                    do controlador de aquisicao */

#define pcInfoReq		0x0700 		/* Mensagem de solicitacao de 
	                                   informacao sobre controlador */
#define pcInfoReq_R		0x0701 		/* Resposta a pcInfoReq */

/* 0x1000 a 0x10ff: reservados para aquisicao de sinais */

#define pcAcqSetup		0x1000 		/* Setup da aquisicao */
#define pcAcqSetupR		0x1001 		/* Resposta para o pcAcqSetup */
#define pcAcqStart		0x1002 		/* Inicia a aquisicao */
#define pcAcqStartR		0x1003 		/* Resposta para o pcAcqStart */
#define pcAcqPause		0x1004 		/* Pausa na aquisicao */
#define pcAcqPauseR		0x1005 		/* Resposta para o pcAcqPause */
#define pcAcqStop 		0x1006 		/* Para a aquisicao (finaliza) */
#define pcAcqStopR		0x1007 		/* Resposta para o pcAcqStop */
#define pcAcqData  		0x1009 		/* Dados da aquisicao */


/*==============================================================*/
/* Estruturas                                                   */
/*==============================================================*/

/*--------------------------------------------------------------*/ 
/* Formato generico de pacotes UDP do AC1160                    */

typedef struct _TpBufMsg {
  word PacketCode;		  	    	/* Codigo do pacote */
  word Reserved;			    	/* Reservado */
  word Data[736];
 } TpBufMsg, *pTpBufMsg;

/*--------------------------------------------------------------*/ 
/* Formato do pacote de solicitacao de leitura de entr. digital */
/* do barramento ADS. Mensagem pcReadDI                         */

typedef struct _TpPcAdsReadDI {
  word PacketCode;				/* Codigo do pacote     */
  word Reserved;				/* reservado            */
  word AddrADS;					/* endereco no barramento ADS */
/*  byte AddrLocal;				 se AddrADS=0, endereco local do controlador AC2122 */
} TpPcAdsReadDI, *pTpPcAdsReadDI;


/*--------------------------------------------------------------*/ 
/* Formato do pacote de resposta ao pcReadDI                    */

typedef struct _TpPcAdsReadDI_R {
  word PacketCode;				/* Codigo do pacote     */
  word Reserved;				/* reservado            */
  word ErLoVLo;					/* Parte Low: ==0 -> OK; != 0 -> erro . Parte High: parte Low do valor lido*/
/*  byte Error;					 0: OK; != 0, erro      */
  word VHi;					    /* Parte Low: parte High do Valor lido na entrada digital */
} TpPcAdsReadDI_R, *pTpPcAdsReadDI_R;


/*--------------------------------------------------------------*/ 
/* Formato do pacote de solicitacao de escrita em saida digital */
/* do barramento ADS. Mensagem pcWriteDO                        */

typedef struct _TpPcAdsWriteDO {
  word PacketCode;		  	    /* Codigo do pacote*/
  word Reserved;			    /* Reservado  */
  word AddrADS;					/* endereco no barramento ADS */
/*  byte AddrLocal;				 se AddrADS=0, endereco local do controlador AC2122 */
  word Value;					/* dado a ser escrito */
} TpPcAdsWriteDO, *pTpPcAdsWriteDO;

/*--------------------------------------------------------------*/ 
/* Formato do pacote de resposta ao pcWriteDO                   */

typedef struct _TpPcAdsWriteDO_R {
  word PacketCode;		  	    	/* Codigo do pacote */
  word Reserved;			    	/* Reservado        */
  byte Error;					/* 0: OK; != 0, erro    */
} TpPcAdsWriteDO_R, *pTpPcAdsWriteDO_R;

/*--------------------------------------------------------------*/ 
/* Formato do pacote de solicitacao de informacao               */
/* sobre o controlador. Mensagem pcInfoReq                      */

typedef struct _TpPcInfoReq {
  word PacketCode;		  	    	/* Codigo do pacote */
  word Reserved;			    	/* Reservado */
  word Request;					/* reservado para uso futuro */
} TpPcInfoReq, *pTpPcInfoReq;

/*--------------------------------------------------------------*/ 
/* Formato do pacote de resposta ao pcInfoReq                   */

typedef struct _TpPcInfoReq_R {
  word PacketCode;		  	    /* Codigo do pacote */
  word Reserved;			    /* Reservado  */
  byte Error;					/* 0: OK; != 0, erro */
  byte Model[16];				/* Modelo do controlador */
  byte Hw_Ver;					/* Versao de Hardware */
  byte Sw_Ver;		 			/* Versao de Software */
  byte Protocol;		 		/* Tipo de protocolo suportado */
  byte Protoc_Ver;		 		/* Versao do protocolo */
} TpPcInfoReq_R, *pTpPcInfoReq_R;


/*--------------------------------------------------------------*/ 
/* Formato das mensagens de resposta aos pacotes pcAcqXXXX --}  */

typedef struct _TpPcAcqResp {
  word PacketCode;		  	    /* Codigo do pacote */
  word Reserved;			    /* Reservado        */
  byte Error;                   /* <> 0 indica erro */
} TpPcAcqResp, *pTpPcAcqResp;

/*--------------------------------------------------------------*/ 
/* Formato da mensagem de configuracao da aquisicao - pcAcqSetup*/

typedef struct _TpPcAcqSetup {
  word PacketCode;		  	    /* Codigo do pacote */
  word Reserved;			    /* Reservado  */

  word Mode;            /* Modo da aquisicao. Descricao dos bits: */
                        /* bit 0: 0 -> sincronismo interno */
                        /*        1 -> sincronismo externo (veja bit 1) */
                        /* bit 1: 0 -> sicronismo com /SYNC_RX em borda _/- */
                        /*        1 -> sicronismo com /SYNC_RX em borda -\_ */
						/* bit 2: 0 -> ignora o sinal /TRIG_RX */
						/*        1 -> aguarda /TRIG_RX (veja bit 3) */
						/* bit 3: 0 -> inicia com /TRIC_RX em borda _/- */
						/*        1 -> inicia com /TRIC_RX em borda -\_ */
						/* bit 4: 0 -> SLAVE  - nao envia TRIG/SAMPLE (SYNC) no SYNCRO BUS */
						/*        1 -> MASTER - envia TRIG/SAMPLE (SYNC) no SYNCRO BUS */
						/* bit 8: 0 -> USA tabela de calibracao na leitura AD */
						/*        1 -> IGNORA tabela de calibracao na leitura AD */


  word TrigMode;        /* reservado */
  word PreScaler;       /* Pre-scaler para o timer da aquisicao (SW)*/
  word ClockDiv;		/* Divisor do clock do timer da aquisicao   */
  long SampleFreq;		/* Frequencia de amostragem (float)         */
  short int  nBurstPack;      /* Numero de bursts por pacote pcAcqData    */
  short int  nSignals;		/* Numero de sinais na memoria de canais    */
  word ac_icm[129];		/* Memoria de canais (formato do ADS):      */
						/*   byte low : canal e faixa de entrada    */
						/*   byte high:	endereco ADS                */
    /* bit 0     : 0=Unipolar, 1=Bipolar  - nao usado nesta versao 		 */
    /* bit 2 e 1 : G1|G0 - 0 0 = ganho X1 								 */
    /*                     0 1 = ganho x2 								 */
    /*                     1 0 = ganho x5 								 */
    /*                     1 1 = ganho x10								 */
    /* bit 6 a 3 : MUX3|MUX2|MUX1|MUX0 - canal 0-15 multiplexado no CAD  */
    /* bit 7     : nao usado nesta versao 								 */
    /* bit 14 a 8: Endereco da placa de expansao 						 */
    /* bit 15    : 0=Entrada Digital, 1=Entrada Analogica 				 */
                        
 } TpPcAcqSetup, *pTpPcAcqSetup;

/*--------------------------------------------------------------*/ 
/* Formato do pacote de pausa da aquisicao - pcAcqPause         */
/* pcAcqPause, pcAcqStop                                        */

typedef struct _TpPcAcqPause {
  word PacketCode;		  	    /* Codigo do pacote */
  word Reserved;			    /* Reservado        */
  byte fPause;					/* 0: tira de pausa */
								/* 1: coloca a aquisicao em pausa (inibe timer da aquisicao) */
} TpPcAcqPause, *pTpPcAcqPause;


#define MAX_AMS_FRAME   720     /* Numero maximo de amostras por frame pcAcqData */


/*--------------------------------------------------------------*/ 
/* Formato do pacote com dados da aquisicaoo - pcAcqData        */
typedef struct _TpPcAcqData {
  word PacketCode;		  	    /* Codigo do pacote */
  word Reserved;			    /* Reservado        */
  dword FrameNo;                /* Numero sequencial do pacote pcAcqData. Deve */
                                /* iniciar com zero */
  long  TimeStamp;              /* Tempo em milisegundos (futuro) */
  long  SampleFreq;             /* Frequencia de amostragem (float). Deve ser */
								/* o mesmo valor recebido no pcAcqSetup */
  short int   nBurst;           /* Numero de bursts no pacote. Deve ser o mesmo */
                                /* recebido em nBurstPack do pcAcqSetup */
  short int   nSignals;         /* Numero de sinais no pacote. Deve ser o mesmo */
								/* recebido no pcAcqSetup */
  word  OwnerIP1_IP2;           /* Endereco IP de quem mandou o pcAcqSetup */
  word  OwnerIP3_IP4;
  word  OwnerPort;				/* Port UDP de quem mandou o pcAcqSetup */
  word  Status;					/* Status    */
  long  ReservedL1;             /* reservado */
  short int Sample[MAX_AMS_FRAME];  /* Amostras aquisitadas */
} TpPcAcqData, *PTpPcAcqData;


/*==============================================================*/
/* Pacotes de acesso aos recursos da AC1160                     */
/*==============================================================*/

#define pcSetIP			0x0002		 /* Programa endereco IP */
#define pcSetIPR    	0x0003       /* Resposta do pcSetIP  */
#define pcSetSecureIP	0x0004		 /* Programa a configuracao e endereco IP */
#define pcSetSecureIPR  0x0005       /* Resposta do pcSetSecureIP  */
#define pcGetSecureIP	0x0006		 /* Leitura da configuracao e endereco IP */
#define pcGetSecureIPR  0x0007       /* Resposta do pcGetSecureIP  */
#define pcGetInfo		0x0008		 /* Solicita informacoes gerais do aparelho */
#define pcGetInfoR  	0x0009       /* Resposta do pcGetInfo  */
#define pcSetFilter		0x000A		 /* Programa a configuracao dos filtros */
#define pcSetFilterR  	0x000B       /* Resposta do pcSetFilter  */
#define pcGetFilter		0x000C		 /* Solicita a configuracao dos filtros */
#define pcGetFilterR  	0x000D       /* Resposta do pcGetFilter  */
#define pcSaveFilter	0x000E		 /* Salva a configuracao dos filtros na EEPROM */
#define pcSaveFilterR  	0x000F       /* Resposta do pcSaveFilter  */
#define pcLoadFilter	0x0010		 /* Carrega configuracao dos filtros na EEPROM */
#define pcLoadFilterR  	0x0011       /* Resposta do pcLoadFilter  */

/*--------------------------------------*/
/* mensagem pcAnswerR                   */

typedef struct _TpPcAnswerR {
  word PacketCode;		  	   /* Codigo do pacote */
  word Reserved;			   /* Reservado  */
  word  Error;            	   /* codigo de erro */
} TpPcAnswerR, *PTpPcAnswerR;

/*--------------------------------------*/
/* mensagem pcSetIP                     */

typedef struct _TpPcSetIP {
  word PacketCode;		  	  /* Codigo do pacote */
  word Reserved;			   	  /* Reservado  */
  word  NewIP1_IP2;         /* novo Endereco IP word mais  signifcativo*/
  word  NewIP3_IP4;			   	/* novo Endereco IP word menos signifcativo*/
  word  ModelCode;			    /* codigo do modelo 1160/1120 */
} TpPcSetIP, *PTpPcSetIP;

/*-----------------------------------------*/
/* mensagem pcSetSecureIP e pcGetSecureIPR */

typedef struct _TpPcSetSecureIP {
  word PacketCode;              /* Codigo do pacote */
  word Reserved;                /* Reservado  */
  word  NewIP1_IP2;             /* novo Endereco IP word mais  signifcativo          */
  word  NewIP3_IP4;			   	/* novo Endereco IP word menos signifcativo          */
  word  DisBlockIP;			   	/* 1/0 : Hab/Inibe restricao de resposta para OwnerIP*/
  word  OwnerIP1_IP2;           /* Endereco IP de quem mandou o pcSetSecureIP        */
  word  OwnerIP3_IP4;
  word  swTag[5];               /* tag do equipamento								*/
} TpPcSetSecureIP, *PTpPcSetSecureIP;

/*--------------------------------------*/
/* mensagem pcGetInfoR                  */

typedef struct _TpPcGetInfoR {
  word PacketCode;              /* Codigo do pacote 	*/
  word Reserved;		   	    /* Reservado  		*/
  word Error;                   /* <> 0 indica erro	*/
  word  DisBlockIP;             /* 1/0 : Hab/Inibe restricao de resposta para OwnerIP*/
  word  OwnerIP1_IP2;           /* Endereco IP de quem mandou o pcSetSecureIP        */
  word  OwnerIP3_IP4;
  word  swTag[5];		        /* tag do equipamento								*/
  word  swModel[8];             /* modelo do equipamento						*/
  word  SerialNoH;              /* numero de serie do equipamento   */
  word  SerialNoL;              /* numero de serie do equipamento   */
  word  NIP1H;                  /* NIP da parte 1 (AC1160)                      */
  word  NIP1L;			        /* NIP da parte 1 (AC1160)                      */
  word  NIP2H;			        /* NIP da parte 2 (condicionador ou expansao)   */
  word  NIP2L;			        /* NIP da parte 2 (condicionador ou expansao)   */
  word  NIP3H;			        /* NIP da parte 1 (placa de conexao)            */
  word  NIP3L;			        /* NIP da parte 1 (placa de conexao)            */
  word  VersionP1;              /* Versao do Hw (LSB) e Sw (MSB) da parte 1		*/
  word  VersionP2;		        /* Versao do Hw (LSB) e Sw (MSB) da parte 2		*/
  word  VersionP3;		        /* Versao do Hw (LSB) e Sw (MSB) da parte 3		*/
  word  CondGain[16];	        /* Ganho dos condicionadores por canal			*/
                                /* formato real compactado Lynx					*/  
  word  ListHPF[16];            /* frequencia de corte HPF						*/
                                /* formato real compactado Lynx					*/  
  word  ListLPF[16];            /* frequencia de corte LPF						*/
			  	                /* formato real compactado Lynx					*/  
  word  Notch;                  /* filtros notch implementados				    */
  			                    /*  bit 05: notch 150Hz                         */
                				/*  bit 06: notch 100Hz                         */
  						        /*  bit 07: notch  50Hz                         */
        						/*  bit 13: notch 180Hz                         */
                                /*  bit 14: notch 120Hz                         */
                                /*  bit 15: notch  60Hz                         */
} TpPcGetInfoR, *PTpPcGetInfoR;

/*--------------------------------------*/
/* mensagem pcSetFilter                 */

typedef struct _TpPcSetFilter {
  word PacketCode;		   /* Codigo do pacote 	*/
  word Reserved;		   /* Reservado  		*/
  word  Error;             /* <> 0 indica erro 	*/
  word  Filter[16];		   /* configuracao dos filtros por canal*/
                           /* filtro HPF no MSB e LPF no LSB 	*/
                           /* bit 00..03: indice da lista do LPF, 15=sem filtro */
                           /* bit 04    : reservado                             */
                           /* bit 05    : habilita notch 150Hz                  */
                           /* bit 06    : habilita notch 100Hz                  */
                           /* bit 07    : habilita notch 50Hz                   */
                           /* bit 08..11: indice da lista do HPF, 16=sem filtro */
                           /* bit 12    : reservado                             */
                           /* bit 13    : habilita notch 180Hz                  */
                           /* bit 14    : habilita notch 120Hz                  */
                           /* bit 15    : habilita notch 60Hz                   */
} TpPcSetFilter, *PTpPcSetFilter;


